from sqlalchemy import Column, func, ForeignKey
from sqlalchemy.orm import relationship
from sqlalchemy.dialects.postgresql import UUID, JSON, TIMESTAMP, TEXT
from uuid import uuid4
from sqlalchemy.ext.declarative import declarative_base
from database_orm.models.tasks import Task

from database_orm.models.task_statuses import TaskStatus
from database_orm.models.task_results import TaskResult


Base = declarative_base()


class TaskRequest(Base):
    __tablename__ = 'task_requests'

    uuid = Column(UUID(as_uuid=True), primary_key=True, default=uuid4)

    task_uuid = Column(UUID(as_uuid=True), ForeignKey(Task.uuid), nullable=False, index=True)
    task = relationship(Task, back_populates="requests")

    result = relationship('TaskResult', uselist=False, back_populates="task_request")

    statuses = relationship('TaskStatus', back_populates="task_request")

    data = Column(JSON)

    callback = Column(TEXT)

    created_at = Column(TIMESTAMP, server_default=func.current_timestamp())
    updated_at = Column(TIMESTAMP, onupdate=func.current_timestamp())

